import * as React from "react";
import { styled } from "@mui/material/styles";
import Table from "@mui/material/Table";
import TableBody from "@mui/material/TableBody";
import TableCell, { tableCellClasses } from "@mui/material/TableCell";
import TableContainer from "@mui/material/TableContainer";
import TableHead from "@mui/material/TableHead";
import TableRow from "@mui/material/TableRow";
import Paper from "@mui/material/Paper";
import {
	Box,
	Pagination,
	TextField,
	Typography,
	InputAdornment,
} from "@mui/material";
import SearchIcon from "@mui/icons-material/Search";
import ArrowUpwardIcon from "@mui/icons-material/ArrowUpward";
import ArrowDownwardIcon from "@mui/icons-material/ArrowDownward";

// Styled Components
const StyledTableCell = styled(TableCell)(({ theme }) => ({
	[`&.${tableCellClasses.head}`]: {
		backgroundColor: "#000000",
		color: "#D0E1E9",
		fontWeight: 600,
		fontSize: "16px",
		padding: "12px 16px",
		borderBottom: "none",
		position: "sticky",
		top: 0,
		zIndex: 10,
		transition: "background-color 0.2s ease",
		"&:hover": {
			backgroundColor: "#2A2A2A",
			cursor: "pointer",
		},
	},
	[`&.${tableCellClasses.body}`]: {
		fontSize: "16px",
		padding: "12px 16px",
		borderBottom: "none",
		color: "#333333",
		transition: "background-color 0.2s ease",
		fontWeight: 500,
		
	},
}));

const StyledTableRow = styled(TableRow)(({ theme }) => ({
	backgroundColor: "#DFECF1",
	transition: "background-color 0.2s ease",
	"&:hover": {
		backgroundColor: "#D0E1E9",
	},
}));

const StyledTableContainer = styled(TableContainer)(({ theme }) => ({
	borderRadius: "12px",
	boxShadow: "0 4px 12px rgba(0, 0, 0, 0.1)",
	"&::-webkit-scrollbar": {
		height: "6px",
	},
	"&::-webkit-scrollbar-track": {
		background: "#D0E1E9",
	},
	"&::-webkit-scrollbar-thumb": {
		background: "#000000",
		borderRadius: "3px",
	},
}));

// Interfaces
export interface Column {
	id: string;
	label: string;
	align?: "left" | "center" | "right";
}

interface Row {
	[key: string]: any;
}

interface CustomTableProps {
	columns: Column[];
	rows: Row[];
	tableType: string;
}

// Sorting State Interface
interface SortConfig {
	key: string;
	direction: "asc" | "desc" | null;
}

const CustomTable: React.FC<CustomTableProps> = ({
	columns,
	rows,
	tableType,
}) => {
	const [searchQuery, setSearchQuery] = React.useState("");
	const [currentPage, setCurrentPage] = React.useState(1);
	const [sortConfig, setSortConfig] = React.useState<SortConfig>({
		key: "",
		direction: null,
	});
	const usersPerPage = 10;

	// Helper Functions
	const getTitleByType = (type: string) => {
		const titles: { [key: string]: string } = {
			backlog: "Student Backlog",
			tools: "Tools",
		};
		return titles[type] || "List";
	};

	const getEmptyMessage = (type: string) => {
		const messages: { [key: string]: string } = {
			backlog: "No students available",
			tools: "No tools available",
		};
		return messages[type] || "No data available";
	};

	const isImage = (value: any) => {
		if (typeof value === "string") {
			return value.startsWith("http") || value.startsWith("data:image");
		}
		return false;
	};

	// Sorting Logic
	const handleSort = (key: string) => {
		setSortConfig((prev) => {
			if (prev.key === key) {
				if (prev.direction === "asc") return { key, direction: "desc" };
				if (prev.direction === "desc") return { key, direction: null };
				return { key, direction: "asc" };
			}
			return { key, direction: "asc" };
		});
	};

	const sortedRows = React.useMemo(() => {
		if (!sortConfig.key || !sortConfig.direction) return [...rows];

		return [...rows].sort((a, b) => {
			const aValue = a[sortConfig.key];
			const bValue = b[sortConfig.key];

			if (aValue == null && bValue == null) return 0;
			if (aValue == null) return sortConfig.direction === "asc" ? 1 : -1;
			if (bValue == null) return sortConfig.direction === "asc" ? -1 : 1;

			if (typeof aValue === "number" && typeof bValue === "number") {
				return sortConfig.direction === "asc"
					? aValue - bValue
					: bValue - aValue;
			}

			const aStr = String(aValue).toLowerCase();
			const bStr = String(bValue).toLowerCase();
			return sortConfig.direction === "asc"
				? aStr.localeCompare(bStr)
				: bStr.localeCompare(aStr);
		});
	}, [rows, sortConfig]);

	// Filtering Logic
	const filteredRows = sortedRows.filter((row) => {
		if (!searchQuery.trim()) return true;
		const lowerCaseQuery = searchQuery.toLowerCase().trim();
		return (
			(row.name && row.name.toLowerCase().includes(lowerCaseQuery)) ||
			(row.email && row.email.toLowerCase().includes(lowerCaseQuery))
		);
	});

	// Pagination Logic
	const indexOfLastRow = currentPage * usersPerPage;
	const indexOfFirstRow = indexOfLastRow - usersPerPage;
	const currentRows = filteredRows.slice(indexOfFirstRow, indexOfLastRow);

	const handlePageChange = (
		_event: React.ChangeEvent<unknown>,
		value: number
	) => {
		setCurrentPage(value);
	};

	return (
		<Box
			sx={{
				width: "100%",
				padding: "20px 8px",
				backgroundColor: "#D0E1E9",
				borderRadius: "12px",
			}}
		>
			<Box
				sx={{
					display: "flex",
					flexDirection: { xs: "column", md: "row" },
					justifyContent: "space-between",
					alignItems: { xs: "flex-start", md: "center" },
					flexWrap: "wrap",
					mb: 3,
					gap: 2,
					
				}}
			>
				<Typography
					sx={{
						fontWeight: 600,
						fontSize: "30px",
						letterSpacing: "-0.5px",
					}}
				>
					{getTitleByType(tableType)}
				</Typography>

				<TextField
					variant="outlined"
					placeholder="Search"
					sx={{
						width: { xs: "100%", sm: "300px" },
						"& .MuiOutlinedInput-root": {
							borderRadius: "8px",
							height: "40px",
							backgroundColor: "#FFFFFF",
							"& fieldset": { borderColor: "#D1D5DB" },
							"&:hover fieldset": { borderColor: "#9CA3AF" },
							"&.Mui-focused fieldset": {
								borderColor: "#3B82F6",
							},
						},
						"& .MuiInputBase-input": {
							fontSize: "14px",
							color: "#1A1A1A",
						},
					}}
					InputProps={{
						startAdornment: (
							<InputAdornment position="start">
								<SearchIcon sx={{ color: "#6B7280" }} />
							</InputAdornment>
						),
					}}
					value={searchQuery}
					onChange={(e) => setSearchQuery(e.target.value)}
				/>
			</Box>

			<StyledTableContainer>
				<Table sx={{ minWidth: "750px", tableLayout: "auto" }}>
					<TableHead>
						<TableRow>
							{columns.map((column) => (
								<StyledTableCell
									key={column.id}
									align={column.align}
									onClick={
										column.id !== "select" &&
										column.id !== "status"
											? () => handleSort(column.id)
											: undefined
									}
									sx={{
										cursor:
											column.id !== "select" &&
											column.id !== "status"
												? "pointer"
												: "default",
									}}
								>
									<Box
										sx={{
											display: "flex",
											alignItems: "center",
											gap: 1,
										}}
									>
										{column.label}
										{column.id !== "select" &&
											column.id !== "status" &&
											sortConfig.key === column.id &&
											sortConfig.direction &&
											(sortConfig.direction === "asc" ? (
												<ArrowUpwardIcon
													sx={{
														fontSize: "16px",
														color: "#FFFFFF",
													}}
												/>
											) : (
												<ArrowDownwardIcon
													sx={{
														fontSize: "16px",
														color: "#FFFFFF",
													}}
												/>
											))}
									</Box>
								</StyledTableCell>
							))}
						</TableRow>
					</TableHead>
					<TableBody>
						{currentRows.length === 0 ? (
							<StyledTableRow>
								<StyledTableCell
									colSpan={columns.length}
									sx={{ textAlign: "center", py: 4 }}
								>
									<Typography
										sx={{
											color: "#6B7280",
											fontSize: "16px",
										}}
									>
										{getEmptyMessage(tableType)}
									</Typography>
								</StyledTableCell>
							</StyledTableRow>
						) : (
							currentRows.map((row, rowIndex) => (
								<StyledTableRow key={rowIndex}>
									{columns.map((column) => (
										<StyledTableCell
											key={column.id}
											align={column.align}
										>
											{isImage(row[column.id]) ? (
												<img
													src={row[column.id]}
													alt="Row"
													style={{
														width: "40px",
														height: "40px",
														borderRadius: "50%",
														objectFit: "cover",
													}}
												/>
											) : (
												row[column.id]
											)}
										</StyledTableCell>
									))}
								</StyledTableRow>
							))
						)}
					</TableBody>
				</Table>
			</StyledTableContainer>

			<Box sx={{ display: "flex", justifyContent: "flex-end", mt: 3 }}>
				<Pagination
					count={Math.ceil(filteredRows.length / usersPerPage)}
					page={currentPage}
					onChange={handlePageChange}
					sx={{
						"& .MuiPaginationItem-root": {
							color: "#1A1A1A",
							fontSize: "16px",
						},
						"& .Mui-selected": {
							backgroundColor: "#000000",
							color: "#FFFFFF",
							"&:hover": {
								backgroundColor: "#000000",
							},
						},
					}}
				/>
			</Box>
		</Box>
	);
};

export default CustomTable;
