import { useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import * as XLSX from "xlsx";
import { v4 as uuidv4 } from "uuid";
import { useNavigate } from "react-router-dom";
import { AppDispatch, RootState } from "../../redux/store/store";
import { loadStudents } from "../../redux/slices/backlogSlice";
import { toast } from "sonner";
import { Box, Button, Typography } from "@mui/material";
import CustomTable, { Column } from "../../components/CustomTable/customTable";

export default function BacklogUploadPage() {
	const dispatch = useDispatch<AppDispatch>();
	const navigate = useNavigate();

	const students = useSelector(
		(state: RootState) => state.backlogReducer.students
	);

	const [fileUploaded, setFileUploaded] = useState(students.length > 0);
	const [selectedStudentIds, setSelectedStudentIds] = useState<string[]>([]);

	const handleFileUpload = (e: React.ChangeEvent<HTMLInputElement>) => {
		const file = e.target.files?.[0];
		if (!file) return;

		const reader = new FileReader();
		reader.onload = (evt) => {
			const data = new Uint8Array(evt.target.result as ArrayBuffer);
			const workbook = XLSX.read(data, { type: "array" });
			const sheet = workbook.Sheets[workbook.SheetNames[0]];
			const parsedData: { name: string; email: string }[] =
				XLSX.utils.sheet_to_json(sheet);

			const studentsWithId = parsedData.map((student) => ({
				...student,
				id: uuidv4(),
				status: "Pending",
			}));

			dispatch(loadStudents(studentsWithId));
			setFileUploaded(true);
			setSelectedStudentIds([]); // Reset selection after upload
		};

		reader.readAsArrayBuffer(file);
	};

	const handleCheckboxChange = (id: string) => {
		setSelectedStudentIds((prev) =>
			prev.includes(id) ? prev.filter((s) => s !== id) : [...prev, id]
		);
	};

	const handleNavigateToEmail = () => {
		if (selectedStudentIds.length === 0) {
			toast.warning("Please select at least one student.");
			return;
		}
		navigate("/email", { state: { selectedIds: selectedStudentIds } });
	};

	const columns: Column[] = [
		{ id: "select", label: "Select", align: "center" },
		{ id: "name", label: "Name", align: "left" },
		{ id: "email", label: "Email", align: "left" },
		{ id: "status", label: "Status", align: "center" },
	];

	const rows = students.map((student) => ({
		id: student.id,
		select: (
			<input
				type="checkbox"
				checked={selectedStudentIds.includes(student.id)}
				onChange={() => handleCheckboxChange(student.id)}
				disabled={student.status === "Sent"}
			/>
		),
		name: student.name,
		email: student.email,
		status:
			student.status === "Sent" ? (
				<span style={{ color: "#10B981" }}>✔️ Sent</span>
			) : (
				<span style={{ color: "#F59E0B" }}>⏳ Pending</span>
			),
	}));

	return (
		<Box
			sx={{
				width: "100vw",
				minHeight: "100vh",
				display: "flex",
				flexDirection: "column",
				p: { xs: 2, sm: 4 },
				backgroundColor: "#F5F8FA",
				boxSizing: "border-box",
			}}
		>
			<Typography
				sx={{
					fontWeight: 600,
					fontSize: { xs: "20px", sm: "24px" },
					mb: 3,
					color: "#1A1A1A",
				}}
			>
				📂 Upload Student Backlog File
			</Typography>

			{!fileUploaded ? (
				<Box sx={{ maxWidth: "600px" }}>
					<label htmlFor="upload-input">
						<input
							id="upload-input"
							type="file"
							accept=".xlsx, .xls"
							onChange={handleFileUpload}
							style={{ display: "none" }}
						/>
						<Button
							variant="contained"
							component="span"
							sx={{
								backgroundColor: "#1F2937",
								color: "#fff",
								borderRadius: "8px",
								textTransform: "none",
								fontWeight: "500",
							}}
						>
							📁 Choose Excel File
						</Button>
					</label>
					<Typography
						sx={{
							fontSize: "14px",
							color: "#6B7280",
							mt: 1,
						}}
					>
						* Excel must contain columns: <b>name</b>, <b>email</b>
					</Typography>
				</Box>
			) : (
				<Box sx={{ flex: 1, display: "flex", flexDirection: "column" }}>
					<CustomTable
						columns={columns}
						rows={rows}
						tableType="backlog"
					/>
					<Box sx={{ display: "flex", gap: 2, mt: 3 }}>
						<Button
							variant="contained"
							onClick={handleNavigateToEmail}
							sx={{
								backgroundColor: "#111827",
								color: "#fff",
								borderRadius: "8px",
								fontSize: "16px",
								padding: "8px 24px",
								textTransform: "none",
								"&:hover": { backgroundColor: "#1F2937" },
							}}
						>
							Send Email
						</Button>
					</Box>
				</Box>
			)}
		</Box>
	);
}
