import { createSlice, createAsyncThunk, PayloadAction } from "@reduxjs/toolkit";
import http from "../../lib/https";
import { toast } from "sonner";

export interface Student {
	id: string;
	name: string;
	email: string;
	status: "Pending" | "Sent";
}

// State shape
interface BacklogState {
	students: Student[];
	isLoading: boolean;
	error: string | null;
}

// Initial state
const initialState: BacklogState = {
	students: [],
	isLoading: false,
	error: null,
};


export const fetchBacklogStudents = createAsyncThunk<
	Student[],
	void,
	{ rejectValue: string }
>("backlog/fetchBacklogStudents", async (_, { rejectWithValue }) => {
	try {
		const response = await http.get("/students/backlog");
		return response.data;
	} catch (error: any) {
		return rejectWithValue(
			error.response?.data?.message || "Failed to fetch students"
		);
	}
});

export const removeStudentFromServer = createAsyncThunk<
	string, // returning student ID
	string, // input student ID
	{ rejectValue: string }
>("backlog/removeStudentFromServer", async (studentId, { rejectWithValue }) => {
	try {
		await http.delete(`/students/${studentId}`);
		toast.success("Student removed from backlog.");
		return studentId;
	} catch (error: any) {
		toast.error("Failed to remove student.");
		return rejectWithValue(
			error.response?.data?.message || "Remove operation failed"
		);
	}
});

const backlogSlice = createSlice({
	name: "backlog",
	initialState,
	reducers: {
		loadStudents: (
			state,
			action: PayloadAction<Omit<Student, "status">[]>
		) => {
			state.students = action.payload.map((student) => ({
				...student,
				status: "Pending",
			}));
			toast.success("Students loaded from file.");
		},


		removeStudent: (state, action: PayloadAction<string>) => {
			state.students = state.students.filter(
				(student) => student.id !== action.payload
			);
			toast.success("Student removed from local backlog.");
		},

		
		markStudentAsSent: (state, action: PayloadAction<string[]>) => {
			action.payload.forEach((id) => {
				const student = state.students.find((s) => s.id === id);
				if (student) {
					student.status = "Sent";
				}
			});
		},
	},

	extraReducers: (builder) => {
		builder
			.addCase(fetchBacklogStudents.pending, (state) => {
				state.isLoading = true;
				state.error = null;
			})
			.addCase(
				fetchBacklogStudents.fulfilled,
				(state, action: PayloadAction<Student[]>) => {
					state.students = action.payload;
					state.isLoading = false;
				}
			)
			.addCase(
				fetchBacklogStudents.rejected,
				(state, action: PayloadAction<string | undefined>) => {
					state.isLoading = false;
					state.error = action.payload || "Failed to fetch";
				}
			)
			.addCase(removeStudentFromServer.fulfilled, (state, action) => {
				state.students = state.students.filter(
					(student) => student.id !== action.payload
				);
			});
	},
});

export const { loadStudents, removeStudent, markStudentAsSent } =
	backlogSlice.actions;
export default backlogSlice.reducer;
